@@ -3,17 +3,17 @@ NULL
 
 #' @export
 .espell <- setRefClass(
-  Class="espell",
-  contains="eutil",
-  methods=list(
-    initialize=function (method, ...) {
+  Class    = "espell",
+  contains = "eutil",
+  methods  = list(
+    initialize = function(method, ...) {
       callSuper()
-      perform_query("espell", method=method, ...)
+      perform_query(method = method, ...)
       if (errors$all_empty()) {
         errors$check_errors(.self)
       }
     },
-    show=function() {
+    show = function() {
       cat("Object of class", sQuote(eutil()), "\n")
       methods::show(get_content("xml"))
     }
@@ -29,17 +29,23 @@ NULL
 #' @return An \code{\linkS4class{espell}} object.
 #' @export
 #' @examples
-#' ###
-espell <- function(term, db="nuccore") {
+#' e <- espell("Chlamidia")
+#' e
+espell <- function(term, db = "pubmed") {
   if (missing(term)) {
-    stop("No query term provided", call.=FALSE)
+    stop("No query term provided", call. = FALSE)
   }
   if (!nzchar(db)) {
-    stop("No database provided", call.=FALSE)
+    stop("No database provided", call. = FALSE)
   }
   if (length(term) > 1L) {
     term <- paste(term, collapse=" OR ")
   }
-  .espell(method=if (nchar(term) < 100) "GET" else "POST",
-           term=.escape(term), db=db)
+  .espell(method = if (nchar(term) < 100) "GET" else "POST",
+          term = .escape(term), db = db, retmode = 'xml')
 }
+
+#' @describeIn content
+setMethod("content", "espell", function(x, as = NULL) {
+  callNextMethod(x = x, as = as)
+})
